/**
 * Un exemple d'utilisation de GtkDrawingArea : dessiner un histogramme.
 *
 * AUTHOR: Guillaume Riviere (C) 2017
 * FILE: callbacks.c
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <gtk/gtk.h>

#include "callbacks.h"

extern GtkBuilder *builder ;

int    G_N ;            /* nombre de valeurs */
int   *G_pourcentages ; /* valeurs */
char **G_legende ;      /* legendes */

G_MODULE_EXPORT void on_window1_realize (GtkObject *object, gpointer user_data) {
  GtkDrawingArea *draw1 ;
  int val, i ;
  char leg[16] ;
  
  printf ("realize\n") ;

  draw1 = GTK_DRAWING_AREA (gtk_builder_get_object (builder, "drawingarea1"));

  /* Definir la taille de la zone de dessin */
  gtk_widget_set_size_request (GTK_WIDGET(draw1), 400, 300) ;

  /* Nombre de rectangle */
  G_N = 5 ; /* > 0 sinon division par zero */

  assert (G_N > 0) ;

  /* Allocation memoire pour les valeurs de pourcentages */
  if ((G_pourcentages = (int *)malloc (G_N * sizeof(int))) == NULL) {
    perror ("Erreur allocation memoire") ;
    exit (1) ;
  }

  /* Allocation memoire pour les etiquettes */
  if ((G_legende = (char **)malloc (G_N * sizeof(char *))) == NULL) {
    perror ("Erreur allocation memoire") ;
    exit (1) ;
  }

  /* Initialisation des valeurs et etiquettes */
  for (val=0, i=0 ; i<G_N ; i++) {

    /* Valeur */
    G_pourcentages[i] = val ;

    /* Legende */
    sprintf (leg, "V%d", i+1) ;
    G_legende[i] = strdup (leg) ;
    
    val += 20 ;
  }
  
  printf ("FIN realize\n") ;
}

G_MODULE_EXPORT void on_window1_destroy (GtkObject *object, gpointer user_data) {
  printf ("Fermeture de la fenetre\n") ;
  gtk_main_quit () ;  /* Stopper la boucle GTK */
}

G_MODULE_EXPORT void on_drawingarea1_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer data) {
  GtkDrawingArea *draw1 ;
  GdkWindow      *win1 ;
  GdkGC          *gc ;
  GdkFont        *font ;
  GdkColor        fg ;
  int W, H ;           /* Taille de la zone de dessin */
  int X0, Y0, W0, H0 ; /* Zone de tracage */
  int x, y, dy, dx, w, h, i ;
  char etiquette[16] ;
  int n = 10 ; /* nombre de lignes de la grille horizontale */
  
  printf ("expose event\n") ;

  draw1 = GTK_DRAWING_AREA (gtk_builder_get_object (builder, "drawingarea1")) ;
  win1 = gtk_widget_get_window (GTK_WIDGET(draw1)) ;
  gc = gdk_gc_new (GDK_DRAWABLE(win1)) ;
  font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*") ;
  
  gtk_widget_get_size_request (GTK_WIDGET (draw1), &W, &H) ;

  X0 = 40 ;
  Y0 = 20 ;
  W0 = W-10 ;
  H0 = H-40 ;
  
  /**
   * See documentation for drawing primitives:
   * https://developer.gnome.org/gdk2/stable/gdk2-Drawing-Primitives.html
   */

  /* Couleur du pinceau (alternative par nom de couleur) */
  gdk_color_parse ("white", &fg) ;
  gdk_gc_set_rgb_fg_color (gc, &fg) ;

  /* Remplir tout la zone en blanc */
  gdk_draw_rectangle (win1, gc, TRUE, 0, 0, W-1, H-1) ;
  
  /* Couleur du pinceau (alternative par nom de couleur) */
  gdk_color_parse ("dark gray", &fg) ;
  gdk_gc_set_rgb_fg_color (gc, &fg) ;
  gdk_gc_set_font (gc, font) ;
  
  /* Dessiner une grille horizontale */

  y = 0 ;
  dy = H0 / (n+1) ;

  for (i=0 ; i<=n ; i++) {

    /* pourcentage vers px */
    y = i*n * H0 / 100.0 ;

    /* etiquette */
    sprintf (etiquette, "%d%%", i*n) ;
    gdk_draw_text (win1, font, gc, 0, Y0+H0-y+3, etiquette, strlen(etiquette)) ;

    /* ligne */
    gdk_draw_line (win1, gc, X0, Y0+H0-y, W0-1, Y0+H0-y) ;
  }
  
  /* Couleur du pinceau (alternative par composante pixels : entre 0 et 65535 ) */
  fg.pixel   = 0 ;
  fg.red     = 0xFFFF ; /* 65535 */
  fg.green   = 0 ;
  fg.blue    = 0xFFFF ; /* 65535 */
  gdk_gc_set_rgb_fg_color (gc, &fg) ;

  /* Dessiner des rectangles, etiquettes et legendes */
  
  dx = 40 ; /* espace entre chaque rectangle */
  w = (W0 - dx*(G_N+1)) / G_N ; /* largeur des rectangles */
  x = X0+X0/2 ;
  y = Y0 ;

  for (i=0 ; i<G_N ; i++) {

    /* pourcentage vers px */
    h = G_pourcentages[i] * H0 / 100.0 ;

    /* etiquette */
    sprintf (etiquette, "%d%%", G_pourcentages[i]) ;
    gdk_draw_text (win1, font, gc, x, Y0+H0-h-3, etiquette, strlen(etiquette)) ;

    /* legende */
    gdk_draw_text (win1, font, gc, x, Y0+H0+13, strdup(G_legende[i]), strlen(G_legende[i])) ;

    /* rectangle */
    gdk_draw_rectangle (win1, gc, TRUE, x, Y0+H0-h, w, h) ;
    x += w + dx ;
  }

  printf ("FIN expose event\n") ;
}

G_MODULE_EXPORT void on_button_plus_clicked (GtkObject *object, gpointer user_data) {
  GtkDrawingArea *draw1 ;

  printf ("button plus clicked\n") ;

  draw1 = GTK_DRAWING_AREA (gtk_builder_get_object (builder, "drawingarea1")) ;
  
  if (G_pourcentages[0] < 100) {
    
    /* Modifier une valeur */
    G_pourcentages[0] += 5 ;
    
    /* Demander que le widget soit redessine */
    gtk_widget_queue_draw (GTK_WIDGET(draw1)) ;
  }

  printf ("FIN button plus clicked\n") ;
}

G_MODULE_EXPORT void on_button_moins_clicked (GtkObject *object, gpointer user_data) {
  GtkDrawingArea *draw1 ;

  printf ("button moins clicked\n") ;

  draw1 = GTK_DRAWING_AREA (gtk_builder_get_object (builder, "drawingarea1")) ;
  
  if (G_pourcentages[0] > 0) {

    /* Modifier une valeur */
    G_pourcentages[0] -= 5 ;
    
    /* Demander que le widget soit redessine */
    gtk_widget_queue_draw (GTK_WIDGET(draw1)) ;
  }

  printf ("FIN button moins clicked\n") ;
}
